function [domindx, permut, permat] = dpdom(a)
%PROGRAM DPDOM - Uses dynamic programming to find a permutation that
%                maximizes the dominance index (sum above main diagonal)
%INPUTS:   a = an n x n asymmetric matrix 
%OUTPUTS   domindx - the optimal dominance index (sum above diagonal)
%          permut - the optimal permutation
%          permat - the optimally permuted matrix
% LIMITATIONS - for n > 21 or 22, the running time could be very slow.
%               The MATLAB implementation is not recommended for n > 25.

tic;
[n,n1] = size(a);                     % Begin dynamic programming
nsum = (2.^n) - 1; bigneg = -99999999;
u = ones(nsum,1).*bigneg;
v = zeros(nsum,1);
for i = 1:n
  index  = 2.^(i-1); summ = 0; q(i) = i;
  for j = 1:n
    if i ~= j
      summ = summ + a(i,j);
    end
  end
  v(index) = i; u(index) = summ;
end
mtc = 0; nh = 0;
for k = 1:n-1
   nfirst = 0;
   if(mtc == 0)
      m2 = 0; 
      for j = 1:k
        w(j) = m2 + j;
      end
      if(w(1) ~= (n-k+1))
         mtc = 1;
      end
   end
 
   while (mtc == 1)
      if(nfirst == 1)
         if(m2 < (n-nh))
           nh = 0;            
         end
         nh = nh + 1; m2 = w(k+1-nh);
         for j = 1:nh
           w(k+j-nh) = m2 + j;
         end
         if w(1) ~= n-k+1
            mtc = 1;
         else
            mtc = 0;
         end
      end
      
      if(nfirst == 0)
         nfirst = 1;
      end
      index = 0;
      for i = 1:k
         index = index + 2^(w(i)-1);
      end
      wcomp = setdiff(q,w);
      nk = n - k;
      for jj = 1:nk
         jone = wcomp(jj);
         summ = 0.0;
         for i = 1:nk
            ione = wcomp(i);
            if ione ~= jone
              summ = summ + a(jone,ione);
            end
         end
         
         temp = u(index) + summ;
         idxtmp = index + 2.^(jone-1);
         comp = u(idxtmp);
         if(temp > comp)
            u(idxtmp) = temp;
            v(idxtmp) = jone;
         end
      end
   end
end
permut(n) = v(nsum);
domindx = u(nsum);
index = nsum;
lastint = permut(n);
for i = 1:(n-1)
   index = index - (2^(lastint-1));
   lastint = v(index);
   permut(n-i) = lastint;
end
permat = a(permut,permut);
toc
      
      
      
      
  


          


