function [robindx, permut, permat] = dprob(a)
%PROGRAM DPROB - Uses dynamic programming to find a permutation that
%                minimizes a QAP-based Robinson criterion.
%INPUTS:   a = an n x n symmetric matrix 
%OUTPUTS   robindx - the optimal value of Robinson index
%          permut - the optimal permutation
%          permat - the optimally permuted matrix
% LIMITATIONS - for n > 21 or 22, the running time could be very slow.
%               The MATLAB implementation is not recommended for n > 25.

tic;
[n,n1] = size(a);                     % Begin dynamic programming
nsum = (2.^n) - 1; bigpos = 99999999;
u = ones(nsum,1).*bigpos;
v = zeros(nsum,1);
d = zeros(n);
for i = 1:n
    for j = 1:n
        d(i,j) = abs(i-j);
        d(j,i) = abs(j-i);
    end
end
for i = 1:n
  index  = 2.^(i-1); summ = 0; q(i) = i;
  for j = 1:n
    if i ~= j
      summ = summ + a(i,j);
    end
  end
  v(index) = i; u(index) = 0;
end
mtc = 0; nh = 0;
for k = 1:n-1
   nfirst = 0;
   if(mtc == 0)
      m2 = 0; 
      for j = 1:k
        w(j) = m2 + j;
      end
      if(w(1) ~= (n-k+1))
         mtc = 1;
      end
   end
 
   while (mtc == 1)
      if(nfirst == 1)
         if(m2 < (n-nh))
           nh = 0;            
         end
         nh = nh + 1; m2 = w(k+1-nh);
         for j = 1:nh
           w(k+j-nh) = m2 + j;
         end
         if w(1) ~= n-k+1
            mtc = 1;
         else
            mtc = 0;
         end
      end
      
      if(nfirst == 0)
         nfirst = 1;
      end
      index = 0;
      for i = 1:k
         index = index + 2^(w(i)-1);
      end
      wcomp = setdiff(q,w);
      nk = n - k;
      rd = d(k,k+1);
      for jj = 1:nk
         jone = wcomp(jj);
         summ = 0.0;
         for kk = 1:nk
             kone = wcomp(kk);
             for i = 1:k
                ione = w(i);
                summ = summ + a(kone,ione).*rd;
            end
         end
         
         temp = u(index) + summ;
         idxtmp = index + 2.^(jone-1);
         comp = u(idxtmp);
         if(temp < comp)
            u(idxtmp) = temp;
            v(idxtmp) = jone;
         end
      end
   end
end
permut(n) = v(nsum);
robindx = u(nsum);
index = nsum;
lastint = permut(n);
for i = 1:(n-1)
   index = index - (2^(lastint-1));
   lastint = v(index);
   permut(n-i) = lastint;
end
permat = a(permut,permut);
toc
      
      
      
      
  


          


