function [domidx, permut, permat] = pidom(A, nreps)
% PIDOM.M - A fast pairwise interchange heuristic for maximizing the
% dominance criterion.
% INPUTS: an n-by-n asymmetric matrix (A) and the number of restarts (nreps)
% OUTPUTS: the best-found dominance index (domidx), the best-found
% permutation (permut), the optimally-permuted matrix (permat).
% SUGGESTIONS: Use when n is too large for DPDOM.M (say n > 25). The
% algortihm is fast, so a good initial recommendation is nreps = 10,000.
[n,n1] = size(A);
tic;
B = zeros(n);
eps = .00000001;
domidx = -999999999999;
for iii = 1:nreps
    s = randperm(n); z = 0;
    for i = 1:n-1
        k = s(i);
        for j = i+1:n
            l = s(j);  z = z + A(k,l);
            B(i,j) = A(j,i) - A(i,j);  B(j,i) = A(i,j) - A(j,i);
        end
    end
    trig = 0;
    while trig == 0
        trig = 1;
        for i1 = 1:n-1
            for j1 = i1+1:n
                k = s(i1); m = s(j1);  delta = B(k,m);
                for l = i1+1:j1-1
                    q = s(l);  delta = delta + B(k,q) + B(q,m);
                end
                if delta > 0
                    z = z + delta; s(i1) = m; s(j1) = k;  trig = 0;
                 end
            end
        end
    end
    if z > domidx
       domidx = z; permut = s;
    end
end
permat = A(permut,permut);
toc;
