function [robidx, permut, permat] = pirob(A, nreps)
% PIROB.M - A fast pairwise interchange heuristic for minimizing the
% Robinson-type QAP criterion.
% INPUTS: an n-by-n symmetric matrix (A) and the number of restarts (nreps)
% OUTPUTS: the best-found Robinson index (robidx), the best-found
% permutation (permut), the optimally-permuted matrix (permat).
% SUGGESTIONS: Use when n is too large for DPROB.M (say n > 25). The
% algortihm is fast, so a good initial recommendation is nreps = 10,000.
[n,n1] = size(A);  D = zeros(n); tic; 
for i = 1:n-1
    for j = i+1:n
        D(i,j) = j-i;  D(j,i) = D(i,j);
    end
end
eps = .00000001; robidx = 999999999999;
for iii = 1:nreps
    s = randperm(n);  z = 0;
    for i = 1:n-1
        k = s(i);
        for j = i+1:n
            l = s(j);  z = z + D(i,j) .* A(k,l);
        end
    end
    trig = 0;
    while trig == 0
        trig = 1;
        for i1 = 1:n-1
            for j1 = i1+1:n
                k = s(i1);  m = s(j1); delta = 0;
                for l1 = 1:n
                    if i1==l1 || j1 == l1
                        continue
                    else
                       l=s(l1);
                       delta = delta + (D(l1,i1)-D(l1,j1))*(A(l,m)-A(l,k));
                    end
                end
                if delta < 0
                    z = z + delta; s(i1) = m; s(j1) = k; trig = 0;
                end
            end
        end
    end
    if z < robidx
       robidx = z; permut = s;
    end
end
permat = A(permut,permut);
toc;
