function [subseq, subsize] = subsequencedom(a, numseq)
% SUBSEQUENCEDOM.M -
% This program will exact the 'numseq' largest subsequences of objects
% such that perfect dominance is achieved for the subsequence.
% INPUTS:  a - which is an n-by-n asymmetric matrix
%          numseq - desired number of subsequences to extract
% OUTPUTS: subseq - a matrix containing the subsequences
%          subsize - a vector containing the size of each subsequence
% SUGGESTIONS:
% (1) Allowing ties is generally OK,; however, for very sparse asymmetric
%     matrices it might be preferable to modify the code to disallow ties
%     by uncommenting the "Dis-allow ties" line.
% (2) numseq = 100 is probably more than sufficient

tic; epsilon=.0000001; [n,n1]=size(a); 
for i = 1:n
  a(i,i) = 0;
end
d = zeros(n,n);
for i = 1:n-1
    for j = i+1:n
        if a(i,j) == a(j,i)
            d(i,j) = 1; d(j,i) = 1;    % Allow ties
%            d(i,j) = 0; d(j,i) = 0;    % Dis-allow ties
        elseif a(i,j) > a(j,i) 
            d(i,j) = 1;
        else
            d(j,i) = 1;
        end
    end
end
subseq = [];
subsize = zeros(numseq,1);
for ijkl = 1:numseq
    maxsize = 0; q = zeros(1,n+1); s = zeros(1,n+1);
    m = 1; q(m) = 1; s(1) = 1; trig1 = 0; trigend = 0;
    while trigend == 0
        if trig1 == 0
            m = m + 1;  trig1 = 1;    % advance pointer
        end
        q(m) = q(m) + 1;
        if s(q(m)) == 1             % redundancy
            continue
        end
        if m == 1 && q(m) > n       % terminate
            trigend = 1;
            continue
        end
        if m > 1 && q(m) > n        % retract
            s(q(m)) = 0; q(m) = 0; m = m - 1; s(q(m)) = 0;
            continue
        end
        test_fail = 0; s(q(m))=1;
        for i = 1:m-1
            if d(q(i),q(m)) == 0
                test_fail = 1; s(q(m))=0;
                break
            end
        end
        if test_fail == 0
            if m > maxsize
                idup = 0;
                for i = 1:ijkl-1
                    if subsize(i) == m
                        ict = 0;
                        for j = 1:m
                            if subseq(i,j) == q(j)
                                ict = ict + 1;
                            end
                        end
                        if ict == m
                            idup = 1;
                        end
                    end
                end
                if idup == 0
                    maxsize = m; selected = q;
                end
            end
            trig1 = 0; 
        end
    end
    subsize(ijkl) = maxsize;
    subseq = [subseq; selected];
end
toc