function [subseq, subsize] = subsequencepath(a, numseq)
% SUBSEQUENCEPATH.M
% This program will extract the 'numseq' largest subsequences of objects
% such that the edge-weights between adjacent pairs of objects in the
% subsequence are monotonically non-increasing (and exceed zero). More
% precisely, for any three consecutive objects (i, j, k) in the subsequence,
% we must have a(i,j) >= a(j,k), and a(j,k) > 0.  
% INPUTS:  a - which is an n-by-n symmetric matrix
%          numseq - desired number of subsequences to extract
% OUTPUTS: subseq - a matrix containing the subsequences
%          subsize - a vector containing the size of each subsequence
% SUGGESTIONS/POSSIBLE MODIFICATIONS:
%     (1) A value of numseq = 100 is typically more than suffificient
%     (2) For monotonically decreasing edge weights, use a(i,j) > a(j,k).
%     (3) It is easy to customize this code to further constrain the
%         subsequences. Later in the code, it is described how to
%         constrain the subsequence to begin with a particular object.
tic; epsilon=.0000001; [n,n1]=size(a); 
for i = 1:n
    a(i,i) = 0;
end
%d = zeros(n,n,n);
%for i = 1:n
%    for j = 1:n
%        if i==j
%            continue
%        end
%        for k = 1:n
%            if k==i || k==j
%                continue
%            end
%            if a(i,j) >= a(i,k) && a(i,k) > 0  % Allow ties - but no zeros          
%              if a(i,j) > a(i,k) && a(i,k) > 0   % Dis-allow ties
%                d(i,j,k) = 1;
%            end
%        end
%    end
%end
subseq = [];
subsize = zeros(numseq,1);
for ijkl = 1:numseq
    maxsize = 0; q = zeros(1,n+1); s = zeros(1,n+1);
    m = 1; q(m) = 1; s(1) = 1; trig1 = 0; trigend = 0;
    while trigend == 0
        if trig1 == 0
            m = m + 1;  trig1 = 1;    % advance pointer
        end
        q(m) = q(m) + 1;
        if s(q(m)) == 1             % redundancy
            continue
        end
        if m == 1 && q(m) > n       % terminate
            trigend = 1;
            continue
        end
        if m > 1 && q(m) > n        % retract
            s(q(m)) = 0; q(m) = 0; m = m - 1; s(q(m)) = 0;
            continue
        end
        test_fail = 0;
        if m > 2
            if a(q(m-2),q(m-1)) < a(q(m-1),q(m)) || a(q(m-1),q(m)) == 0   || q(1) ~= 8
 % NOTE: If you uncomment "|| q(1) ~= 8" in the line above, then only
 %       subsequences beginning with symptom 8 will be allowed
                test_fail = 1;
            end
        end
        if test_fail == 0
            if m > maxsize
                idup = 0;
                for i = 1:ijkl-1
                    if subsize(i) == m
                        ict = 0;
                        for j = 1:m
                            if subseq(i,j) == q(j)
                                ict = ict + 1;
                            end
                        end
                        if ict == m
                            idup = 1;
                        end
                    end
                end
                if idup == 0
                    maxsize = m; selected = q;
                end
            end
            trig1 = 0;  s(q(m))=1;
        end
    end
    subsize(ijkl) = maxsize;
    subseq = [subseq; selected];
end
toc
